/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Module Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <string.h>

#include <ostore/ostore.hh>
#include <ostore/coll.hh>
#include <ostore/relat.hh>
#include "OO7.h"
#include "GenParams.h"
#include "VarParams.h"

extern os_database* oo7db;

extern int	nextComplexAssemblyId;
extern int	debugMode;
extern char*	types[NumTypes];

////////////////////////////////////////////////////////////////////////////
//
// Module Constructor
//
////////////////////////////////////////////////////////////////////////////

Module::Module(int modId) : id(modId)
{
    if (debugMode) {
        printf("Module::Module(modId = %d)\n", modId);
    }

    // initialize the new module's data members (some randomly)

    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinModuleDate +
    		(int) (random() % (MaxModuleDate - MinModuleDate + 1));

    // put the manual with the other documents, in a separate segment
    man = new (os_segment::of(Document::extent)) Manual (modId);

    // add module to Module extent

    (*Module::extent) |= this;

    // now create the assemblies for the module

    int assmId;

    if (NumAssmLevels > 1) {
        assmId = nextComplexAssemblyId++;
        ComplexAssembly* assm = new (os_segment::of(this)) ComplexAssembly
	    			(assmId, NULL, 1, this);
        designRoot = assm;
    }
}


////////////////////////////////////////////////////////////////////////////
//
// Module Destructor
//
////////////////////////////////////////////////////////////////////////////

Module::~Module()
{
    if (debugMode) {
        printf("Module::~Module(id = %d)\n", id);
    }

    // remove module from Module extent
    (*Module::extent) -= this;
}


///////////////////////////////////////////////////////////////////////////
//
// Module::scanManual() returns the number of occurrences
// of the character 'I' in the module's manual.
//
///////////////////////////////////////////////////////////////////////////

int Module::scanManual()
{
    return man->searchText('I');
}


///////////////////////////////////////////////////////////////////////////
//
// Module::firstLast() returns one if the first and last character
// of the manual are the same, zero otherwise.
//
///////////////////////////////////////////////////////////////////////////

int Module::firstLast()
{
    return man->firstLast();
}

